@@ -1,5 +1,6 @@
 #!/usr/local/bin/perl -l
 use strict;
+use lib qw(../lib/ lib/);
 use AI::Prolog qw/:all/;
 
 my $database = Parser->consult(<<'END_PROLOG');
@@ -9,6 +10,7 @@ steals(PERP, X) :-
 END_PROLOG
 my $query = Term->new("steals(badguy,X).");
 my $engine = Engine->new($query, $database);
+$engine->formatted(1);
 print $engine->results;
 
 $query = Term->new("steals(ovid, X).");
